--[=[
Recruiter World of Warcraft Add-on
Copyright (c) 2015 by Eric Isakson (eric.david.isakson@gmail.com), All rights reserved

Version r20151126184015

Recruiter is a tool for coordinating recruitment activities accross guildmates.
It is intended to help provide a consistent message in recruiting activities and
prevent spamming community members with invitations. It is also useful
for managing guild membership with tools for removing inactive members and 
banning players so they are not re-invited by a recruiter after being kicked
from the guild.

This add-on uses the Ace3 framework.

type /recruiter to get a list of existing options.

-----
    Announcer.lua
-----

This component announces to other recruiters that you are initiating contact
with a lead.


--]=]

local ERROR     = 1;
local WARNING   = 2;
local INFO      = 3;
local INFO2     = 4;


local ADDON_NAME, T = ...;
local RECRUITER_ = T.Recruiter;
local L = RECRUITER_.Localized_Text;

-- Create module
RECRUITER_.Announcer = RECRUITER_:NewModule("Announcer");
local Announcer = RECRUITER_.Announcer;

-- Up Values
local _G              = _G;
local SendChatMessage = _G.SendChatMessage;
local GetTime         = _G.GetTime;
local UnitName        = _G.UnitName;
local db              = RECRUITER_.db;
local SetHandler      = RECRUITER_.SetHandler;
local ColorText       = RECRUITER_.ColorText;
local ReplaceTokens   = RECRUITER_.ReplaceTokens;
local ValidateMessage = RECRUITER_.ValidateMessage;

function Announcer:OnInitialize() -- {{{
    self:Debug(INFO, "OnInitialize called!");
    self.db = db:RegisterNamespace('Announcer', {
        global = {
            Announcer = false,
            Throttle = 2 * 60,
            Message = false,
            Channel = 'AUTO',
        },
    });
end -- }}}

function Announcer:GetOptions () -- {{{

    local validateMessage = function (info, v)
        return ValidateMessage(v, {'LEAD'}, 16)
    end

    return {
        [Announcer:GetName()] = {
            name = L[Announcer:GetName()],
            type = 'group',
            get = function (info) return Announcer.db.global[info[#info]]; end,
            set = function (info, value) SetHandler(self, info, value) end,
            args = {
                Announcer = {
                    type = 'toggle',
                    name = L["ANNOUNCER_OPT"],
                    desc = L["ANNOUNCER_OPT_DESC"],
                    order = 10,
                },
                AnnouncerOptions = {
                    type = 'group',
                    inline = true,
                    name = L['ANNOUNCER_OPT_SETTINGS'],
                    hidden = function() return Announcer.db.global.Announcer == false end,
                    order = 40,
                    args = {
                        Description = {
                            type = 'description',
                            order = 0,
                            name = L["ANNOUNCER_OPT_DESCRIPTION"],
                        },
                        Channel = {
                            type = 'select',
                            name = L['ANNOUNCER_OPT_CHANNEL'],
                            desc = L['ANNOUNCER_OPT_CHANNEL_DESC'],
                            values = { ['AUTO'] = L['CHANNEL_AUTO_OFFICER_GUILD'], ['OFFICER'] = L['CHANNEL_OFFICER'], ['GUILD'] = L['CHANNEL_GUILD']},
                            order = 30,
                        },
                        -- throttle TODO move throttling to throttle lead contacting rather than limiting announcing
                        Throttle = { 
                            type = 'range',
                            min = 60,
                            max = 10 * 60,
                            step = 1,
                            bigStep = 5,
                            name = L["ANNOUNCER_OPT_THROTTLE"],
                            desc = L["ANNOUNCER_OPT_THROTTLE_DESC"],
                            order = 40,
                        },
                        -- TODO this doesn't seem quite right...need to read about option validation
                        ValidityCheck = {
                            type = 'description',
                            order = 45,
                            name = ColorText(L["ANNOUNCER_ERROR_MESSAGE_ISSUE"],'FFFF4040'),
                            hidden = function ()
                                if Announcer.db.global.Announcer == false or Announcer.db.global.Message then
                                    return true;
                                else
                                    return false;
                                end
                            end
                        },
                        Message = {
                            type = 'input',
                            width = 'full',
                            name = L["ANNOUNCER_OPT_MESSAGE"],
                            desc = L["ANNOUNCER_OPT_MESSAGE_DESC"],
                            get = function (info)
                                return Announcer.db.global[info[#info]] or L["ANNOUNCER_OPT_MESSAGE_DEFAULT"];
                            end,
                            validate = validateMessage,
                            order = 60,
                        },
                    },
                },
            },
        },
    };
end -- }}}


function Announcer:OnEnable() -- {{{
    self:Debug(INFO, "OnEnable");

    -- Subscribe to RECRUITER_ callbacks
    self:RegisterMessage("RECRUITER_CONTACTING_LEAD");
end -- }}}

function Announcer:OnDisable() -- {{{
    self:Debug(INFO2, "OnDisable");
    self:UnregisterMessage("RECRUITER_CONTACTING_LEAD");
end -- }}}

function Announcer:RECRUITER_CONTACTING_LEAD(selfevent, leadProfile)
    self:Announce(leadProfile)
end

do

    local function GetDistributionChannel()
        local channel = Announcer.db.global.Channel;

        if channel ~= 'AUTO' then
            return channel;
        end

        -- if we are allowed to speak in officer chat
        -- TODO how do we check this and what channel name to return

        -- if we are allowed to speak in guild chat
        -- TODO how do we check this and what channel name to return

        -- if you can't talk in guild why are you able to invite???
        -- fallback to whisper self
        return "WHISPER";
    end

    local function Post(text)
        local channelType = GetDistributionChannel();

        --  SendChatMessage("msg" [,"type" [,"lang" [,"channel"] ] ]).
        SendChatMessage("RECRUITER: " .. text, channelType, nil, channelType == 'WHISPER' and (UnitName('player')) or nil);
    end

    local LastAnnounce = 0;
    function Announcer:Announce(leadProfile)
        -- check lead
        if not leadProfile then
            self:Error(L["ANNOUNCER_ERROR_NO_PROFILE"]);
            return false;
        end
        
        -- check config
        if not (self.db.global.Announcer and self.db.global.Message) then
            self:Error(L["ANNOUNCER_ERROR_FEATURE_NOT_CONFIGURED"]);
            return false;
        end
        local config = self.db.global;

        -- check throttle
        if GetTime() - LastAnnounce < config.Throttle then
            self:Error(L["ANNOUNCER_ERROR_TOO_SOON_WAIT"]);
            return false;
        end

        -- send to chat
        local RecruitersText = ReplaceTokens(config.Message, leadProfile);
        self:Debug(INFO, "RECRUITER_ANNOUNCER:", RecruitersText);
        Post(RecruitersText);
        
        -- log the time to prevent spam
        LastAnnounce = GetTime();

        return true;
    end
end
