--[=[
Recruiter World of Warcraft Add-on
Copyright (c) 2015 by Eric Isakson (eric.david.isakson@gmail.com), All rights reserved

Version r20151126184015

Recruiter is a tool for coordinating recruitment activities accross guildmates.
It is intended to help provide a consistent message in recruiting activities and
prevent spamming community members with invitations. It is also useful
for managing guild membership with tools for removing inactive members and 
banning players so they are not re-invited by a recruiter after being kicked
from the guild.

This add-on uses the Ace3 framework.

type /recruiter to get a list of existing options.

-----
    Targeting.lua
-----

This component plays alert sounds and display messages when you target
or mouseover a lead.


--]=]

local ERROR     = 1;
local WARNING   = 2;
local INFO      = 3;
local INFO2     = 4;


local ADDON_NAME, T = ...;
local RECRUITER_ = T.Recruiter;
local L = RECRUITER_.Localized_Text;

-- Create module
RECRUITER_.Targeting = RECRUITER_:NewModule("Targeting");
local Targeting = RECRUITER_.Targeting;

-- Up Values
local _G               = _G;
local UnitGUID         = _G.UnitGUID;
local UnitIsUnit       = _G.UnitIsUnit;
local PlaySoundFile    = _G.PlaySoundFile;
local db               = RECRUITER_.db;
local SetHandler       = RECRUITER_.SetHandler;
local GetLeadProfile   = RECRUITER_.GetLeadProfile;
local GetClassHexColor = RECRUITER_.GetClassHexColor;
local Print            = RECRUITER_.Print;

function Targeting:OnInitialize() -- {{{
    self:Debug(INFO, "OnInitialize called!");
    self.db = db:RegisterNamespace('Targeting', {
        global = {
            Targeting = true,

            ChatMessages = true,
            Sounds = true,
            Solo = true,
            Party = false,
            Raid = false,
        },
    });
end -- }}}

function Targeting:GetOptions () -- {{{
    return {
        [Targeting:GetName()] = {
            name = L[Targeting:GetName()],
            type = 'group',
            get = function (info) return Targeting.db.global[info[#info]]; end,
            set = function (info, value) SetHandler(self, info, value) end,
            args = {
                Targeting = {
                    type = 'toggle',
                    name = L["TARGETING_OPT"],
                    desc = L["TARGETING_OPT_DESC"],
                    order = 1,
                },
                TargetingOptions = {
                    type = 'group',
                    inline = true,
                    name = L['TARGETING_OPT_SETTINGS'],
                    hidden = function() return Targeting.db.global.Targeting == false end,
                    order = 40,
                    args = {
                        Description = {
                            type = 'description',
                            order = 0,
                            name = L["TARGETING_OPT_DESCRIPTION"],
                        },
                        ChatMessages = {
                            type = 'toggle',
                            name = L["TARGETING_OPT_CHAT_MESSAGES"],
                            desc = L["TARGETING_OPT_CHAT_MESSAGES_DESC"],
                            order = 10,
                        },
                        Sounds = {
                            type = 'toggle',
                            name = L["TARGETING_OPT_SOUNDS"],
                            desc = L["TARGETING_OPT_SOUNDS_DESC"],
                            order = 10,
                        },
                        Solo = {
                            type = 'toggle',
                            name = L["TARGETING_OPT_SOLO"],
                            desc = L["TARGETING_OPT_SOLO_DESC"],
                            order = 30,
                        },
                        Party = {
                            type = 'toggle',
                            name = L["TARGETING_OPT_PARTY"],
                            desc = L["TARGETING_OPT_PARTY_DESC"],
                            order = 30,
                        },
                        Raid = {
                            type = 'toggle',
                            name = L["TARGETING_OPT_RAID"],
                            desc = L["TARGETING_OPT_RAID_DESC"],
                            order = 30,
                        },
                    },
                },
            },
        },
    };
end -- }}}


function Targeting:OnEnable() -- {{{
    self:Debug(INFO, "OnEnable");

    self:RegisterEvent("UPDATE_MOUSEOVER_UNIT", "TestUnit");
    self:RegisterEvent("PLAYER_TARGET_CHANGED", "TestUnit");
    
    -- TODO add solo/party/raid detection and adjust listeners based on options
    -- for now this is always on when this module is enabled
end -- }}}

function Targeting:OnDisable() -- {{{
    self:Debug(INFO2, "OnDisable");
    
    self:UnregisterEvent("UPDATE_MOUSEOVER_UNIT");
    self:UnregisterEvent("PLAYER_TARGET_CHANGED");
end -- }}}

-- MouseOver and Target trigger {{{
do
    local LastDetectedGUID = "";
    function Targeting:TestUnit(eventName)

        local unit="";
        if eventName=="UPDATE_MOUSEOVER_UNIT" then
            unit = "mouseover";
        elseif eventName=="PLAYER_TARGET_CHANGED" then
            unit = "target";
        else
            self:Debug(ERROR, "called on invalid event");
            return;
        end

        local unitGuid = UnitGUID(unit);
        if not unitGuid then
            --self:Debug(WARNING, "No unit GUID");
            return;
        end

        local leadProfile = GetLeadProfile(unitGuid)
        if leadProfile then

            if LastDetectedGUID == unitGuid and unit == "target" then
                self:LeadTargetted(leadProfile);
                return;
            end

            if not UnitIsUnit("mouseover", "target") and unit == "mouseover" then
                self:LeadMousedOver(leadProfile);
                LastDetectedGUID = unitGuid;
            end
        end
        
    end
end -- }}}

local previousUnitGuid;
function Targeting:LeadMousedOver(leadProfile)
    if previousUnitGuid ~= leadProfile.guid then
        self:RecruitThatLead(leadProfile, "Sound\\interface\\AlarmClockWarning3.ogg");
        previousUnitGuid = leadProfile.guid;
    end
end

function Targeting:LeadTargetted(leadProfile)
    self:RecruitThatLead(leadProfile, "Sound\\interface\\AuctionWindowOpen.ogg");
end


function Targeting:RecruitThatLead(leadProfile, soundFile)
    self:PlaySoundFile(soundFile);
    if self.db.global.ChatMessages then
        local subjectColor = GetClassHexColor(leadProfile.class);
        Print(L["TARGETING_RECRUIT_THAT_LEAD"]:format("|c" .. subjectColor, leadProfile.name));
    end
end

function Targeting:PlaySoundFile(...) -- {{{
    if self.db.global.Sounds then
        PlaySoundFile(...);
    end
end -- }}}
