--[=[
Recruiter World of Warcraft Add-on
Copyright (c) 2015 by Eric Isakson (eric.david.isakson@gmail.com), All rights reserved

Version r20151126184015

Recruiter is a tool for coordinating recruitment activities accross guildmates.
It is intended to help provide a consistent message in recruiting activities and
prevent spamming community members with invitations. It is also useful
for managing guild membership with tools for removing inactive members and 
banning players so they are not re-invited by a recruiter after being kicked
from the guild.

This add-on uses the Ace3 framework.

type /recruiter to get a list of existing options.

-----
    utils.lua
-----


--]=]

local ERROR     = 1;
local WARNING   = 2;
local INFO      = 3;
local INFO2     = 4;


local ADDON_NAME, T = ...;
local RECRUITER = T.Recruiter;

local RECRUITER_C = T.Recruiter.Constants;

local _G        = _G;
local pairs     = _G.pairs;
local ipairs    = _G.ipairs;
local tostring  = _G.tostring;
local table     = _G.table;
local select    = _G.select;
local type      = _G.type;
local date      = _G.date;
local debugstack= _G.debugstack;
local string_gsub = _G.string.gsub;

function RECRUITER:MakePlayerName (name) --{{{
    if not name then name = "NONAME" end
    return "|Hplayer:" .. name .. "|h" .. (name):upper() .. "|h";
end --}}}

function RECRUITER:ValidateMessage (message, keywords, minLength)
        for i, keyword in ipairs(keywords) do
            if not v:find('%['..keyword..'%]') then
                return self:Error(L["ERROR_MESSAGE_MISSING_KEYWORD"]);
            end
        end

        if v:len() < minLength then
            return self:Error(L["ERROR_MESSAGE_TOO_SHORT"]);
        end

        return 0, v;
end
    
function RECRUITER:ColorText (text, color) --{{{

    if type(text) ~= "string" then
        text = tostring(text)
    end

    return "|c".. color .. text .. "|r";
end --}}}


-- Class coloring related functions {{{
local RAID_CLASS_COLORS = _G.RAID_CLASS_COLORS;

RECRUITER_C.ClassesColors = { };

local LC = _G.LOCALIZED_CLASS_NAMES_MALE;

function RECRUITER:GetClassColor (englishClass) -- {{{
    if not RECRUITER_C.ClassesColors[englishClass] then
        if RAID_CLASS_COLORS and RAID_CLASS_COLORS[englishClass] then
            RECRUITER_C.ClassesColors[englishClass] = { RAID_CLASS_COLORS[englishClass].r, RAID_CLASS_COLORS[englishClass].g, RAID_CLASS_COLORS[englishClass].b };
        else
            RECRUITER_C.ClassesColors[englishClass] = { 0.63, 0.63, 0.63 };
        end
    end
    return unpack(RECRUITER_C.ClassesColors[englishClass]);
end -- }}}

RECRUITER_C.HexClassColor = { };

function RECRUITER:GetClassHexColor(englishClass) -- {{{

    if not RECRUITER_C.HexClassColor[englishClass] then

        local r, g, b = self:GetClassColor(englishClass);

        RECRUITER_C.HexClassColor[englishClass] = ("FF%02x%02x%02x"):format( r * 255, g * 255, b * 255);

    end

    return RECRUITER_C.HexClassColor[englishClass];
end -- }}}

function RECRUITER:CreateClassColorTables () -- {{{
    if RAID_CLASS_COLORS then
        local class, colors;
        for class in pairs(RAID_CLASS_COLORS) do
            if LC[class] then -- thank to a wonderful add-on that adds the wrong translation "Death Knight" to the global RAID_CLASS_COLORS....
                RECRUITER:GetClassHexColor(class);
            else
                RAID_CLASS_COLORS[class] = nil; -- Eat that!
                self:Print("|cFFFF0000Stupid value found in _G.RAID_CLASS_COLORS table|r\nThis will cause many issues (tainting), RECRUITER will display this message until the culprit add-on is fixed or removed, the Stupid value is: '", class, "'");
            end
        end
    else
        RECRUITER:Debug(ERROR, "global RAID_CLASS_COLORS does not exist...");
    end
end -- }}}
-- }}}

local UIErrorsFrame = _G.UIErrorsFrame
local UIERRORS_HOLD_TIME = _G.UIERRORS_HOLD_TIME
function RECRUITER:Error(message)
    UIErrorsFrame:AddMessage("RECRUITER: " .. message, 1, 0, 0, 1, UIERRORS_HOLD_TIME);
    self:Print(RECRUITER:ColorText(message, 'FFFF3030'));
    return message;
end

-- function RECRUITER:UnitName(Unit) {{{
local UnitName = _G.UnitName;
function RECRUITER:UnitName(Unit)
    local name, server = UnitName(Unit);
        if ( server and server ~= "" ) then
            return name.."-"..server;
        else
            return name;
        end 
end
-- }}}

local GetGuildInfo = _G.GetGuildInfo;
function RECRUITER:ReplaceTokens(message, lead)
    self:Debug(INFO, "RECRUITER:utils:ReplaceTokens:entering ", message, lead.name);
    
    local guild = select(1,GetGuildInfo("player"));
    self:Debug(INFO, "RECRUITER:utils:ReplaceTokens:replacing %[GUILD%] with: ", guild);
    self:Debug(INFO, "RECRUITER:utils:ReplaceTokens:replacing %[LEAD%] with: ", lead.name or 'unknown');
    local replaced = string_gsub(string_gsub(message, '%[LEAD%]', lead.name or 'unknown'), '%[GUILD%]', guild);
    self:Debug(INFO, "RECRUITER:utils:ReplaceTokens:returning: ", replaced);
    return replaced;
end

-- function RECRUITER:RotateTexture(self, degrees) {{{
local mrad = _G.math.rad;
local mcos = _G.math.cos;
local msin = _G.math.sin;
-- inspired from http://www.wowwiki.com/SetTexCoord_Transformations#Simple_rotation_of_square_textures_around_the_center
function RECRUITER:RotateTexture(self, degrees)
	local angle = mrad(degrees)
	local cos, sin = mcos(angle), msin(angle)
        self:SetTexCoord(
        0.5-sin, 0.5+cos,
        0.5+cos, 0.5+sin,
        0.5-cos, 0.5-sin,
        0.5+sin, 0.5-cos
        );
end -- }}}

--  function RECRUITER:Debug(...) {{{
do
    local Debug_Templates = {
        [ERROR]     = "|cFFFF2222Debug:|cFFCC4444[%0.3f]:|r|cFFFF5555",--3
        [WARNING]   = "|cFFFF2222Debug:|cFFCC4444[%0.3f]:|r|cFF55FF55",--2
        [INFO]      = "|cFFFF2222Debug:|cFFCC4444[%0.3f]:|r|cFF9999FF",--1
        [INFO2]     = "|cFFFF2222Debug:|cFFCC4444[%0.3f]:|r|cFFFF9922",--1
        [false]     = "|cFFFF2222Debug:|cFFCC4444[%0.3f]:|r",
    }
    local Debug_Levels = {
        [INFO2]     = 1,
        [INFO]      = 1,
        [WARNING]   = 2,
        [ERROR]     = 3,
        [false]     = 3,
    }
    local select, type, debugprofilestop = _G.select, _G.type, _G.debugprofilestop;

    local timerStart = debugprofilestop();
    function RECRUITER:Debug(...)

        -- if Decursive is loaded then use its debug report facility...
        if (...) == ERROR and DecursiveRootTable then
            local message = {'RECRUITER Debug error:', select(2, ...)};
            message[#message + 1] = '\nSTACK:\n' .. debugstack(2);
            DecursiveRootTable._RecruiterErrors = DecursiveRootTable._RecruiterErrors + 1;
            DecursiveRootTable._AddDebugText(unpack(message));
        end

        if not RECRUITER.db.global.Debug then return end;

        local template = type((...)) == "number" and (...) or false;

        if RECRUITER.db.global.DebugLevel and RECRUITER.db.global.DebugLevel > Debug_Levels[template] then
            return;
        end

        local DebugHeader = (Debug_Templates[template]):format((debugprofilestop() - timerStart) / 1000);

        if template then
            self:Print(DebugHeader, select(2, ...));
        else
            self:Print(DebugHeader, ...);
        end
    end
end -- }}}

-- function RECRUITER:GetOPtionPath(info) {{{
function RECRUITER:GetOPtionPath(info)
    return table.concat(info, "->");
end -- }}}

-- Iterators

function RECRUITER:pairs_ordered (t, reverse, SortKey) -- Not to be used where performance and memory are important

    if not reverse then
        reverse = 1;
    end

    local sortedTable_Keys  = {};


    local i = 1;
    -- store the keys
    for key, value in pairs(t) do

        if SortKey and not value[SortKey] then
            self:Debug(ERROR, "listOrderedIter(): invalid SortKey", SortKey, key, value, value.name);
            error("listOrderedIter(): invalid SortKey");

            return nil;
        end

        sortedTable_Keys[i] = key;
        i = i + 1;
    end


    local function sort (a, b)
        if SortKey then
            if reverse then
                return t[a][SortKey] > t[b][SortKey];
            else
                return t[a][SortKey] < t[b][SortKey];
            end
        else
            if reverse then
                return t[a] > t[b];
            else
                return t[a] < t[b];
            end
        end
    end

    -- sort the keys according to the given parameters
    table.sort(sortedTable_Keys, sort);

    i = 0;
    -- return an iterator
    return function ()
        i = i + 1;

        if sortedTable_Keys[i] then
            return sortedTable_Keys[i], t[sortedTable_Keys[i]];
        end

    end, t, 0;

end

function RECRUITER:AddDelayedFunctionCall(callID, functionLink, ...)

    
    if (not self.DelayedFunctionCalls[callID]) then 
        self.DelayedFunctionCalls[callID] =  {["func"] = functionLink, ["args"] =  {...}};
        self.DelayedFunctionCallsCount = self.DelayedFunctionCallsCount + 1;
    elseif select("#",...) > 1 then -- if we had more than the function reference and its object

        local args = self.DelayedFunctionCalls[callID].args;

        for i=1,select("#",...), 1 do
            args[i]=select(i, ...);
        end

    end
end

local function BadLocalTest (localtest)
        RECRUITER:Print(RECRUITER.Localized_Text[localtest]);
end

function RECRUITER:MakeError(something)

    local testlocal = "test local";
    local testbiglocal = RECRUITER;

    if something == 1 then
        -- Make something forbidden
        TargetUnit('player');
        return;
    elseif something == 2 then
        BadLocalTest("Bad local");
        return;
    end

    local errorf = function () testErrorCapturing(testlocal); end;

    errorf();
end

--[===[@debug@
function RECRUITER:Hickup(mul)
    if not mul then mul = 1 end
    local t = 0;

    for i=1, mul * 1000000, 1 do
        t = t + 1
    end

    self:Debug(WARNING, 'Hickup ', t);
end
--@end-debug@]===]


function RECRUITER:FatalError (TheError)

    if not StaticPopupDialogs["RECRUITER_ERROR_FRAME"] then
        StaticPopupDialogs["RECRUITER_ERROR_FRAME"] = {
            text = "|cFFFF0000Recruiter Fatal Error:|r\n%s",
            button1 = "OK",
            OnAccept = function()
                return false;
            end,
            timeout = 0,
            whileDead = 1,
            hideOnEscape = 1,
            showAlert = 1,
            preferredIndex = 3,
        };
    end

    StaticPopup_Show ("RECRUITER_ERROR_FRAME", TheError);
end

function RECRUITER:GetBAddon (StackLevel)
    local stack = debugstack(1 + StackLevel,2,0);
    if not stack:lower():find("\\libs\\")
        and not stack:find("[/\\]CallbackHandler")
        and not stack:find("[/\\]AceTimer")
        and not stack:find("[/\\]AceHook")
        and not stack:find("[/\\]AceEvent") then

        if stack:find("[/\\]Recruiter") then
            self:Debug(ERROR, "GetBAddon failed!"); -- XXX to test
            return false;
        end

        return stack:match("[/\\]AddOns[/\\]([^/\\]+)[/\\]"), stack;
    else
        self:Debug(WARNING, 'SetScript called but not reported:', stack);
        return false;
    end
end
